@@ -154,21 +154,27 @@ defaults = {
 
             'toolbar': False,
             'new_episodes': 'show',  # ignore, show, queue, download
+            'only_added_are_new': False,  # Only just added episodes are considered new after an update
             'live_search_delay': 200,
             'search_always_visible': False,
+            'find_as_you_type': True,
 
             'podcast_list': {
-                'all_episodes': True,
-                'sections': True,
                 'view_mode': 1,
                 'hide_empty': False,
+                'all_episodes': True,
+                'sections': True,
             },
 
             'episode_list': {
-                'descriptions': True,
                 'view_mode': 1,
-                'columns': int('110', 2),  # bitfield of visible columns
                 'always_show_new': True,
+                'trim_title_prefix': True,
+                'descriptions': True,
+                'show_released_time': False,
+                'right_align_released_column': False,
+                'ctrl_click_to_sort': False,
+                'columns': int('110', 2),  # bitfield of visible columns
             },
 
             'download_list': {
@@ -187,9 +193,12 @@ defaults = {
         'one_folder_per_podcast': True,
         'skip_played_episodes': True,
         'delete_played_episodes': False,
+        'delete_deleted_episodes': False,
 
         'max_filename_length': 120,
 
+        'compare_episode_filesize': True,
+
         'custom_sync_name': '{episode.sortdate}_{episode.title}',
         'custom_sync_name_enabled': False,
 
@@ -203,6 +212,7 @@ defaults = {
             'two_way_sync': False,
             'use_absolute_path': True,
             'folder': 'Playlists',
+            'extension': 'm3u',
         }
 
     },
@@ -223,31 +233,6 @@ defaults = {
     },
 }
 
-# The sooner this goes away, the better
-gPodderSettings_LegacySupport = {
-    'limit_rate': 'limit.bandwidth.enabled',
-    'limit_rate_value': 'limit.bandwidth.kbps',
-    'max_downloads_enabled': 'limit.downloads.enabled',
-    'max_downloads': 'limit.downloads.concurrent',
-    'episode_old_age': 'auto.cleanup.days',
-    'auto_remove_played_episodes': 'auto.cleanup.played',
-    'auto_remove_unfinished_episodes': 'auto.cleanup.unfinished',
-    'auto_remove_unplayed_episodes': 'auto.cleanup.unplayed',
-    'max_episodes_per_feed': 'limit.episodes',
-    'show_toolbar': 'ui.gtk.toolbar',
-    'episode_list_descriptions': 'ui.gtk.episode_list.descriptions',
-    'podcast_list_view_all': 'ui.gtk.podcast_list.all_episodes',
-    'podcast_list_sections': 'ui.gtk.podcast_list.sections',
-    'episode_list_view_mode': 'ui.gtk.episode_list.view_mode',
-    'podcast_list_view_mode': 'ui.gtk.podcast_list.view_mode',
-    'podcast_list_hide_boring': 'ui.gtk.podcast_list.hide_empty',
-    'episode_list_columns': 'ui.gtk.episode_list.columns',
-    'auto_cleanup_downloads': 'ui.gtk.download_list.remove_finished',
-    'auto_update_feeds': 'auto.update.enabled',
-    'auto_update_frequency': 'auto.update.frequency',
-    'auto_download': 'ui.gtk.new_episodes',
-}
-
 logger = logging.getLogger(__name__)
 
 
@@ -317,7 +302,7 @@ class Config(object):
         if callback not in self.__observers:
             self.__observers.append(callback)
         else:
-            logger.warn('Observer already added: %s', repr(callback))
+            logger.warning('Observer already added: %s', repr(callback))
 
     def remove_observer(self, callback):
         """
@@ -326,7 +311,7 @@ class Config(object):
         if callback in self.__observers:
             self.__observers.remove(callback)
         else:
-            logger.warn('Observer not added: %s', repr(callback))
+            logger.warning('Observer not added: %s', repr(callback))
 
     def all_keys(self):
         return self.__json_config._keys_iter()
@@ -371,10 +356,11 @@ class Config(object):
 
         if os.path.exists(self.__filename):
             try:
-                data = open(self.__filename, 'rt').read()
+                with open(self.__filename, 'rt') as f:
+                    data = f.read()
                 new_keys_added = self.__json_config._restore(data)
             except:
-                logger.warn('Cannot parse config file: %s',
+                logger.warning('Cannot parse config file: %s',
                         self.__filename, exc_info=True)
                 new_keys_added = False
 
@@ -406,9 +392,6 @@ class Config(object):
         self.schedule_save()
 
     def __getattr__(self, name):
-        if name in gPodderSettings_LegacySupport:
-            name = gPodderSettings_LegacySupport[name]
-
         return getattr(self.__json_config, name)
 
     def __setattr__(self, name, value):
@@ -416,9 +399,6 @@ class Config(object):
             object.__setattr__(self, name, value)
             return
 
-        if name in gPodderSettings_LegacySupport:
-            name = gPodderSettings_LegacySupport[name]
-
         setattr(self.__json_config, name, value)
 
     def migrate_defaults(self):
@@ -427,3 +407,13 @@ class Config(object):
             logger.debug("setting config.device_sync.max_filename_length=120"
                          " (999 is bad for NTFS and ext{2-4})")
             self.device_sync.max_filename_length = 120
+
+    def clamp_range(self, name, min, max):
+        value = getattr(self, name)
+        if value < min:
+            setattr(self, name, min)
+            return True
+        if value > max:
+            setattr(self, name, max)
+            return True
+        return False
