@@ -162,13 +162,14 @@ class UIConfig(config.Config):
 
         if -1 not in (cfg.x, cfg.y, cfg.width, cfg.height):
             # get screen resolution
-            screen = Gdk.Screen.get_default()
-            screen_width = 0
-            screen_height = 0
-            for i in range(0, screen.get_n_monitors()):
-                monitor = screen.get_monitor_geometry(i)
-                screen_width += monitor.width
-                screen_height += monitor.height
+            def get_screen_size(display):
+                monitor_geometries = [display.get_monitor(i).get_geometry() for i in range(display.get_n_monitors())]
+                x0 = min(r.x for r in monitor_geometries)
+                y0 = min(r.y for r in monitor_geometries)
+                x1 = max(r.x + r.width for r in monitor_geometries)
+                y1 = max(r.y + r.height for r in monitor_geometries)
+                return x1 - x0, y1 - y0
+            screen_width, screen_height = get_screen_size(Gdk.Display.get_default())
             logger.debug('Screen %d x %d' % (screen_width, screen_height))
             # reset window position if more than 50% is off-screen
             half_width = cfg.width / 2
@@ -192,8 +193,10 @@ class UIConfig(config.Config):
             # positions (instead using a user-defined placement algorithm) and honor
             # requests after the window has already been shown.
             # Move it a second time after the window has been shown.
-            # The first move reduces chance of window jumping.
-            util.idle_add(window.move, cfg.x, cfg.y)
+            # The first move reduces chance of window jumping,
+            # and gives the window manager a position to unmaximize to.
+            if not cfg.maximized:
+                util.idle_add(window.move, cfg.x, cfg.y)
 
         # Ignore events while we're connecting to the window
         self.__ignore_window_events = True
@@ -201,14 +204,18 @@ class UIConfig(config.Config):
         # Get window state, correct size comes from window.get_size(),
         # see https://developer.gnome.org/SaveWindowState/
         def _receive_configure_event(widget, event):
-            x_pos, y_pos = widget.get_position()
-            width_size, height_size = widget.get_size()
-            maximized = bool(event.window.get_state() & Gdk.WindowState.MAXIMIZED)
-            if not self.__ignore_window_events and not maximized:
-                cfg.x = x_pos
-                cfg.y = y_pos
-                cfg.width = width_size
-                cfg.height = height_size
+            if not self.__ignore_window_events:
+                # TODO: The maximize event might arrive after the configure event.
+                # This causes the maximized size to be saved, and restoring the
+                # window will not save its smaller size. Delaying the save with
+                # idle_add() is not enough time for the state event to arrive.
+                if not bool(event.window.get_state() & Gdk.WindowState.MAXIMIZED):
+                    x_pos, y_pos = widget.get_position()
+                    width_size, height_size = widget.get_size()
+                    cfg.x = x_pos
+                    cfg.y = y_pos
+                    cfg.width = width_size
+                    cfg.height = height_size
 
         window.connect('configure-event', _receive_configure_event)
 
