@@ -17,7 +17,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-from gi.repository import Gdk, GdkPixbuf, Gtk
+from gi.repository import Gdk, Gio, Gtk
 
 import gpodder
 from gpodder import util
@@ -32,11 +32,11 @@ class gPodderChannel(BuilderWidget):
     def new(self):
         self.show_on_cover_load = True
 
-        self.gPodderChannel.set_title(self.channel.title)
-        self.entryTitle.set_text(self.channel.title)
+        self.gPodderChannel.set_transient_for(self.parent_widget)
+        self.title_label.set_text(self.channel.title)
         self.labelURL.set_text(self.channel.url)
-        self.cbSkipFeedUpdate.set_active(self.channel.pause_subscription)
-        self.cbEnableDeviceSync.set_active(self.channel.sync_to_mp3_player)
+        self.skip_feed_update_switch.set_active(self.channel.pause_subscription)
+        self.enable_device_sync_switch.set_active(self.channel.sync_to_mp3_player)
 
         self.section_list = Gtk.ListStore(str)
         active_index = 0
@@ -64,27 +64,40 @@ class gPodderChannel(BuilderWidget):
         self.combo_strategy.set_active(active_index)
 
         self.LabelDownloadTo.set_text(self.channel.save_dir)
-        self.LabelWebsite.set_text(self.channel.link)
+        self.website_label.set_markup('<a href="{}">{}</a>'.format(
+            self.channel.link, self.channel.link)
+            if self.channel.link else '')
+        self.website_label.connect('activate-link', lambda label, url: util.open_website(url))
 
         if self.channel.auth_username:
             self.FeedUsername.set_text(self.channel.auth_username)
         if self.channel.auth_password:
             self.FeedPassword.set_text(self.channel.auth_password)
 
+        # Cover image
+        ag = Gio.SimpleActionGroup()
+        open_cover_action = Gio.SimpleAction.new("openCover", None)
+        open_cover_action.connect('activate', self.on_open_cover_activate)
+        ag.add_action(open_cover_action)
+        refresh_cover_action = Gio.SimpleAction.new("refreshCover", None)
+        refresh_cover_action.connect('activate', self.on_refresh_cover_activate)
+        ag.add_action(refresh_cover_action)
+        self.main_window.insert_action_group("channel", ag)
+
+        cover_menu = Gio.Menu()
+        cover_menu.append("Change cover image", "channel.openCover")
+        cover_menu.append("Refresh image", "channel.refreshCover")
+
+        self.cover_menubutton.set_menu_model(cover_menu)
+
         self.cover_downloader.register('cover-available', self.cover_download_finished)
         self.cover_downloader.request_cover(self.channel)
 
-        # Hide the website button if we don't have a valid URL
-        if not self.channel.link:
-            self.btn_website.hide()
-
-        b = Gtk.TextBuffer()
         if self.channel._update_error:
             err = '\n\n' + (_('ERROR: %s') % self.channel._update_error)
         else:
             err = ''
-        b.set_text(util.remove_html_tags(self.channel.description) + err)
-        self.channel_description.set_buffer(b)
+        self.channel_description.set_text(util.remove_html_tags(self.channel.description) + err)
 
         # Add Drag and Drop Support
         flags = Gtk.DestDefaults.ALL
@@ -95,19 +108,25 @@ class gPodderChannel(BuilderWidget):
         border = 6
         size = self.MAX_SIZE + border * 2
         self.imgCover.set_size_request(size, size)
-        self.imgCoverEventBox.connect('button-press-event',
-                self.on_cover_popup_menu)
+
+        # Title save button state
+        self.title_save_button_saves = True
+
+        self._config.connect_gtk_window(self.gPodderChannel, 'channel_editor', True)
 
         gpodder.user_extensions.on_ui_object_available('channel-gtk', self)
 
         result = gpodder.user_extensions.on_channel_settings(self.channel)
         if result:
             for label, callback in result:
-                self.notebookChannelEditor.append_page(callback(self.channel), Gtk.Label(label))
+                sw = Gtk.ScrolledWindow()
+                sw.add(callback(self.channel))
+                sw.show_all()
+                self.notebookChannelEditor.append_page(sw, Gtk.Label(label))
 
     def on_button_add_section_clicked(self, widget):
         text = self.show_text_edit_dialog(_('Add section'), _('New section:'),
-            affirmative_text=Gtk.STOCK_ADD)
+            affirmative_text=_('_Add'))
 
         if text is not None:
             for index, (section,) in enumerate(self.section_list):
@@ -118,34 +137,13 @@ class gPodderChannel(BuilderWidget):
             self.section_list.append([text])
             self.combo_section.set_active(len(self.section_list) - 1)
 
-    def on_cover_popup_menu(self, widget, event):
-        if not event.triggers_context_menu():
-            return
-
-        menu = Gtk.Menu()
-
-        item = Gtk.MenuItem.new_with_mnemonic(_('_Open'))
-        item.connect('activate', self.on_btnDownloadCover_clicked)
-        menu.append(item)
-
-        item = Gtk.MenuItem.new_with_mnemonic(_('_Refresh'))
-        item.connect('activate', self.on_btnClearCover_clicked)
-        menu.append(item)
-
-        menu.attach_to_widget(widget)
-        menu.show_all()
-        menu.popup(None, None, None, None, event.button, event.time)
-
-    def on_btn_website_clicked(self, widget):
-        util.open_website(self.channel.link)
-
-    def on_btnDownloadCover_clicked(self, widget):
+    def on_open_cover_activate(self, action, *args):
         dlg = Gtk.FileChooserDialog(
             title=_('Select new podcast cover artwork'),
             parent=self.gPodderChannel,
             action=Gtk.FileChooserAction.OPEN)
-        dlg.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
-        dlg.add_button(Gtk.STOCK_OPEN, Gtk.ResponseType.OK)
+        dlg.add_button(_('_Cancel'), Gtk.ResponseType.CANCEL)
+        dlg.add_button(_('_Open'), Gtk.ResponseType.OK)
 
         if dlg.run() == Gtk.ResponseType.OK:
             url = dlg.get_uri()
@@ -154,7 +152,7 @@ class gPodderChannel(BuilderWidget):
 
         dlg.destroy()
 
-    def on_btnClearCover_clicked(self, widget):
+    def on_refresh_cover_activate(self, action, *args):
         self.clear_cover_cache(self.channel.url)
         self.cover_downloader.replace_cover(self.channel, custom_url=False)
 
@@ -162,7 +160,7 @@ class gPodderChannel(BuilderWidget):
         def set_cover(channel, pixbuf):
             if self.channel == channel:
                 if pixbuf is not None:
-                    self.imgCover.set_from_pixbuf(self.scale_pixbuf(pixbuf))
+                    self.imgCover.set_from_pixbuf(util.scale_pixbuf(pixbuf, self.MAX_SIZE))
                 if self.show_on_cover_load:
                     self.main_window.show()
                     self.show_on_cover_load = False
@@ -191,26 +189,61 @@ class gPodderChannel(BuilderWidget):
     def on_gPodderChannel_destroy(self, widget, *args):
         self.cover_downloader.unregister('cover-available', self.cover_download_finished)
 
-    def scale_pixbuf(self, pixbuf):
+    # Title editing callbacks
+    def on_title_edit_button_clicked(self, button):
+        self.title_save_button_saves = True
+        self.title_save_button.set_label(_("_Save"))
+        self.title_stack.set_visible_child(self.title_edit_box)
+        self.title_entry.set_text(self.title_label.get_text())
+        self.title_entry.grab_focus()
 
-        # Resize if width is too large
-        if pixbuf.get_width() > self.MAX_SIZE:
-            f = float(self.MAX_SIZE) / pixbuf.get_width()
-            (width, height) = (int(pixbuf.get_width() * f), int(pixbuf.get_height() * f))
-            pixbuf = pixbuf.scale_simple(width, height, GdkPixbuf.InterpType.BILINEAR)
+    def on_title_entry_changed(self, entry):
+        if len(entry.get_text()) > 0:
+            self.title_save_button_saves = True
+            self.title_save_button.set_label(_("_Save"))
+        else:
+            self.title_save_button_saves = False
+            self.title_save_button.set_label(_("Cancel"))
 
-        # Resize if height is too large
-        if pixbuf.get_height() > self.MAX_SIZE:
-            f = float(self.MAX_SIZE) / pixbuf.get_height()
-            (width, height) = (int(pixbuf.get_width() * f), int(pixbuf.get_height() * f))
-            pixbuf = pixbuf.scale_simple(width, height, GdkPixbuf.InterpType.BILINEAR)
+    def on_title_entry_icon_press(self, entry, icon_pos, *args):
+        self.title_entry.set_text("")
 
-        return pixbuf
+    def on_title_save_button_clicked(self, button):
+        if self.title_save_button_saves:
+            self.title_label.set_text(self.title_entry.get_text())
+        self.title_stack.set_visible_child(self.title_box)
+
+    def on_feed_url_copy_button_clicked(self, button):
+        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+        clipboard.set_text(self.channel.url, -1)
+
+    def on_open_folder_button_clicked(self, button):
+        util.gui_open(self.channel.save_dir, gui=self)
+
+    def on_row_activated(self, listbox, row, *args):
+        # Find the correct widget in the row to activate
+        def _do(w, *args):
+            if w.get_name().startswith("no_activation"):
+                return
+            elif isinstance(w, Gtk.Box):
+                w.foreach(_do)
+            elif isinstance(w, Gtk.ComboBox):
+                w.popup()
+            elif isinstance(w, Gtk.Entry):
+                w.grab_focus()
+            elif isinstance(w, Gtk.Switch):
+                w.set_state(not w.get_state())
+            elif isinstance(w, Gtk.Button):
+                w.emit("clicked")
+        row.foreach(_do)
+
+    def on_btnCancel_clicked(self, widget, *args):
+        self.main_window.destroy()
 
     def on_btnOK_clicked(self, widget, *args):
-        self.channel.pause_subscription = self.cbSkipFeedUpdate.get_active()
-        self.channel.sync_to_mp3_player = self.cbEnableDeviceSync.get_active()
-        self.channel.rename(self.entryTitle.get_text())
+        self.channel.pause_subscription = self.skip_feed_update_switch.get_state()
+        self.channel.sync_to_mp3_player = self.enable_device_sync_switch.get_state()
+        self.channel.rename(self.title_label.get_text())
         self.channel.auth_username = self.FeedUsername.get_text().strip()
         self.channel.auth_password = self.FeedPassword.get_text()
 
