@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2018 The gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,26 +17,18 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-import gtk
+from gi.repository import Gtk
 
 import gpodder
+from gpodder.gtkui.interface.common import BuilderWidget
 
 _ = gpodder.gettext
 
-from gpodder.gtkui.interface.common import BuilderWidget
 
 class gPodderWelcome(BuilderWidget):
-    PADDING = 10
 
     def new(self):
-        for widget in self.vbox_buttons.get_children():
-            for child in widget.get_children():
-                if isinstance(child, gtk.Alignment):
-                    child.set_padding(self.PADDING, self.PADDING,
-                        self.PADDING, self.PADDING)
-                else:
-                    child.set_padding(self.PADDING, self.PADDING)
+        self.gPodderWelcome.set_transient_for(self.parent_widget)
 
     def on_btnCancel_clicked(self, button):
-        self.main_window.response(gtk.RESPONSE_CANCEL)
-
+        self.main_window.response(Gtk.ResponseType.CANCEL)
