@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2018 The gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,18 +17,15 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-import gtk
-import pango
-import cgi
+from gi.repository import Gtk, Pango
 
 import gpodder
+from gpodder import util
+from gpodder.gtkui.interface.common import BuilderWidget, TreeViewHelper
 
 _ = gpodder.gettext
 N_ = gpodder.ngettext
 
-from gpodder import util
-
-from gpodder.gtkui.interface.common import BuilderWidget
 
 class gPodderEpisodeSelector(BuilderWidget):
     """Episode selection dialog
@@ -52,30 +49,27 @@ class gPodderEpisodeSelector(BuilderWidget):
                           checked state if no other value is set
                           (default is False)
       - columns: List of (name, sort_name, sort_type, caption) pairs for the
-                 columns, the name is the attribute name of the episode to be 
-                 read from each episode object.  The sort name is the 
+                 columns, the name is the attribute name of the episode to be
+                 read from each episode object.  The sort name is the
                  attribute name of the episode to be used to sort this column.
                  If the sort_name is None it will use the attribute name for
                  sorting.  The sort type is the type of the sort column.
                  The caption attribute is the text that appear as column caption
                  (default is [('title_markup', None, None, 'Episode'),])
       - title: (optional) The title of the window + heading
-      - instructions: (optional) A one-line text describing what the 
+      - instructions: (optional) A one-line text describing what the
                       user should select / what the selection is for
-      - stock_ok_button: (optional) Will replace the "OK" button with
-                         another GTK+ stock item to be used for the
-                         affirmative button of the dialog (e.g. can 
-                         be gtk.STOCK_DELETE when the episodes to be
-                         selected will be deleted after closing the 
-                         dialog)
-      - selection_buttons: (optional) A dictionary with labels as 
+      - ok_button: (optional) Will replace the "OK" button label with this
+                   string (e.g. can be '_Delete' when the episodes to be
+                   selected will be deleted after closing the dialog)
+      - selection_buttons: (optional) A dictionary with labels as
                            keys and callbacks as values; for each
                            key a button will be generated, and when
                            the button is clicked, the callback will
                            be called for each episode and the return
                            value of the callback (True or False) will
                            be the new selected state of the episode
-      - size_attribute: (optional) The name of an attribute of the 
+      - size_attribute: (optional) The name of an attribute of the
                         supplied episode objects that can be used to
                         calculate the size of an episode; set this to
                         None if no total size calculation should be
@@ -85,16 +79,20 @@ class gPodderEpisodeSelector(BuilderWidget):
                            the supplied episode objects that holds
                            the text for the tooltips when hovering
                            over an episode (default is 'description')
-                           
     """
     COLUMN_INDEX = 0
     COLUMN_TOOLTIP = 1
     COLUMN_TOGGLE = 2
     COLUMN_ADDITIONAL = 3
 
-    def new( self):
+    def new(self):
+        self.gPodderEpisodeSelector.set_transient_for(self.parent_widget)
+        if hasattr(self, 'title'):
+            self.gPodderEpisodeSelector.set_title(self.title)
+
         self._config.connect_gtk_window(self.gPodderEpisodeSelector, 'episode_selector', True)
-        if not hasattr( self, 'callback'):
+
+        if not hasattr(self, 'callback'):
             self.callback = None
 
         if not hasattr(self, 'remove_callback'):
@@ -106,106 +104,103 @@ class gPodderEpisodeSelector(BuilderWidget):
         if not hasattr(self, 'remove_finished'):
             self.remove_finished = None
 
-        if not hasattr( self, 'episodes'):
+        if not hasattr(self, 'episodes'):
             self.episodes = []
 
-        if not hasattr( self, 'size_attribute'):
+        if not hasattr(self, 'size_attribute'):
             self.size_attribute = 'file_size'
 
         if not hasattr(self, 'tooltip_attribute'):
-            self.tooltip_attribute = 'description'
+            self.tooltip_attribute = '_text_description'
 
-        if not hasattr( self, 'selection_buttons'):
+        if not hasattr(self, 'selection_buttons'):
             self.selection_buttons = {}
 
-        if not hasattr( self, 'selected_default'):
+        if not hasattr(self, 'selected_default'):
             self.selected_default = False
 
-        if not hasattr( self, 'selected'):
-            self.selected = [self.selected_default]*len(self.episodes)
+        if not hasattr(self, 'selected'):
+            self.selected = [self.selected_default] * len(self.episodes)
 
         if len(self.selected) < len(self.episodes):
-            self.selected += [self.selected_default]*(len(self.episodes)-len(self.selected))
+            self.selected += [self.selected_default] * (len(self.episodes) - len(self.selected))
 
-        if not hasattr( self, 'columns'):
+        if not hasattr(self, 'columns'):
             self.columns = (('title_markup', None, None, _('Episode')),)
 
-        if hasattr(self, 'title'):
-            self.gPodderEpisodeSelector.set_title(self.title)
-
-        if hasattr( self, 'instructions'):
-            self.labelInstructions.set_text( self.instructions)
+        if hasattr(self, 'instructions'):
+            self.labelInstructions.set_text(self.instructions)
             self.labelInstructions.show_all()
 
-        if hasattr(self, 'stock_ok_button'):
-            if self.stock_ok_button == 'gpodder-download':
-                self.btnOK.set_image(gtk.image_new_from_stock(gtk.STOCK_GO_DOWN, gtk.ICON_SIZE_BUTTON))
-                self.btnOK.set_label(_('Download'))
+        if hasattr(self, 'ok_button'):
+            if self.ok_button == 'gpodder-download':
+                self.btnOK.set_image(Gtk.Image.new_from_icon_name('go-down', Gtk.IconSize.BUTTON))
+                self.btnOK.set_label(_('_Download'))
             else:
-                self.btnOK.set_label(self.stock_ok_button)
-                self.btnOK.set_use_stock(True)
+                self.btnOK.set_image(None)
+                self.btnOK.set_label(self.ok_button)
 
         # check/uncheck column
-        toggle_cell = gtk.CellRendererToggle()
-        toggle_cell.connect( 'toggled', self.toggle_cell_handler)
-        toggle_column = gtk.TreeViewColumn('', toggle_cell, active=self.COLUMN_TOGGLE)
+        toggle_cell = Gtk.CellRendererToggle()
+        toggle_cell.connect('toggled', self.toggle_cell_handler)
+        toggle_column = Gtk.TreeViewColumn('', toggle_cell, active=self.COLUMN_TOGGLE)
         toggle_column.set_clickable(True)
         self.treeviewEpisodes.append_column(toggle_column)
-        
+
         next_column = self.COLUMN_ADDITIONAL
         for name, sort_name, sort_type, caption in self.columns:
-            renderer = gtk.CellRendererText()
+            renderer = Gtk.CellRendererText()
             if next_column < self.COLUMN_ADDITIONAL + 1:
-                renderer.set_property('ellipsize', pango.ELLIPSIZE_END)
-            column = gtk.TreeViewColumn(caption, renderer, markup=next_column)
+                renderer.set_property('ellipsize', Pango.EllipsizeMode.END)
+            column = Gtk.TreeViewColumn(caption, renderer, markup=next_column)
             column.set_clickable(False)
-            column.set_resizable( True)
+            column.set_resizable(True)
             # Only set "expand" on the first column
             if next_column < self.COLUMN_ADDITIONAL + 1:
                 column.set_expand(True)
             if sort_name is not None:
-                column.set_sort_column_id(next_column+1)
+                column.set_sort_column_id(next_column + 1)
             else:
                 column.set_sort_column_id(next_column)
-            self.treeviewEpisodes.append_column( column)
+            self.treeviewEpisodes.append_column(column)
             next_column += 1
-            
+
             if sort_name is not None:
                 # add the sort column
-                column = gtk.TreeViewColumn()
+                column = Gtk.TreeViewColumn()
                 column.set_clickable(False)
                 column.set_visible(False)
-                self.treeviewEpisodes.append_column( column)
+                self.treeviewEpisodes.append_column(column)
                 next_column += 1
 
-        column_types = [ int, str, bool ]
+        column_types = [int, str, bool]
         # add string column type plus sort column type if it exists
         for name, sort_name, sort_type, caption in self.columns:
             column_types.append(str)
             if sort_name is not None:
                 column_types.append(sort_type)
-        self.model = gtk.ListStore( *column_types)
+        self.model = Gtk.ListStore(*column_types)
 
         tooltip = None
-        for index, episode in enumerate( self.episodes):
+        for index, episode in enumerate(self.episodes):
             if self.tooltip_attribute is not None:
                 try:
                     tooltip = getattr(episode, self.tooltip_attribute)
                 except:
                     tooltip = None
-            row = [ index, tooltip, self.selected[index] ]
+            row = [index, tooltip, self.selected[index]]
             for name, sort_name, sort_type, caption in self.columns:
                 if not hasattr(episode, name):
                     row.append(None)
                 else:
-                    row.append(getattr( episode, name))
-                    
+                    row.append(getattr(episode, name))
+
                 if sort_name is not None:
                     if not hasattr(episode, sort_name):
                         row.append(None)
                     else:
-                        row.append(getattr( episode, sort_name))
-            self.model.append( row)
+                        row.append(getattr(episode, sort_name))
+            self.model.append(row)
 
         if self.remove_callback is not None:
             self.btnRemoveAction.show()
@@ -222,8 +217,9 @@ class gPodderEpisodeSelector(BuilderWidget):
         self.episode_list_can_tooltip = True
 
         self.treeviewEpisodes.connect('button-press-event', self.treeview_episodes_button_pressed)
-        self.treeviewEpisodes.set_rules_hint( True)
-        self.treeviewEpisodes.set_model( self.model)
+        self.treeviewEpisodes.connect('popup-menu', self.treeview_episodes_button_pressed)
+        self.treeviewEpisodes.set_rules_hint(True)
+        self.treeviewEpisodes.set_model(self.model)
         self.treeviewEpisodes.columns_autosize()
 
         # Focus the toggle column for Tab-focusing (bug 503)
@@ -241,7 +237,7 @@ class gPodderEpisodeSelector(BuilderWidget):
         (x_bin, y_bin) = treeview.get_bin_window().get_position()
         y -= x_bin
         y -= y_bin
-        (path, column, rx, ry) = treeview.get_path_at_pos(x, y) or (None,)*4
+        (path, column, rx, ry) = treeview.get_path_at_pos(x, y) or (None,) * 4
 
         if not self.episode_list_can_tooltip or column != treeview.get_columns()[1]:
             self.last_tooltip_episode = None
@@ -258,9 +254,12 @@ class gPodderEpisodeSelector(BuilderWidget):
             self.last_tooltip_episode = index
 
             description = util.remove_html_tags(description)
+            # Bug 1825: make sure description is a unicode string,
+            # so it may be cut correctly on UTF-8 char boundaries
+            description = util.convert_bytes(description)
             if description is not None:
                 if len(description) > 400:
-                    description = description[:398]+'[...]'
+                    description = description[:398] + '[...]'
                 tooltip.set_text(description)
                 return True
             else:
@@ -269,71 +268,76 @@ class gPodderEpisodeSelector(BuilderWidget):
         self.last_tooltip_episode = None
         return False
 
-    def treeview_episodes_button_pressed(self, treeview, event):
-        if event.button == 3:
-            menu = gtk.Menu()
+    def treeview_episodes_button_pressed(self, treeview, event=None):
+        if event is None or event.triggers_context_menu():
+            menu = Gtk.Menu()
 
             if len(self.selection_buttons):
                 for label in self.selection_buttons:
-                    item = gtk.MenuItem(label)
+                    item = Gtk.MenuItem(label)
                     item.connect('activate', self.custom_selection_button_clicked, label)
                     menu.append(item)
-                menu.append(gtk.SeparatorMenuItem())
+                menu.append(Gtk.SeparatorMenuItem())
 
-            item = gtk.MenuItem(_('Select all'))
+            item = Gtk.MenuItem(_('Select _all'))
             item.connect('activate', self.on_btnCheckAll_clicked)
             menu.append(item)
 
-            item = gtk.MenuItem(_('Select none'))
+            item = Gtk.MenuItem(_('Select _none'))
             item.connect('activate', self.on_btnCheckNone_clicked)
             menu.append(item)
 
             menu.show_all()
-            # Disable tooltips while we are showing the menu, so 
+            # Disable tooltips while we are showing the menu, so
             # the tooltip will not appear over the menu
             self.episode_list_can_tooltip = False
             menu.connect('deactivate', lambda menushell: self.episode_list_allow_tooltips())
-            menu.popup(None, None, None, event.button, event.time)
+            if event is None:
+                func = TreeViewHelper.make_popup_position_func(treeview)
+                menu.popup(None, None, func, None, 3, Gtk.get_current_event_time())
+            else:
+                menu.popup(None, None, None, None, event.button, event.time)
 
             return True
 
     def episode_list_allow_tooltips(self):
         self.episode_list_can_tooltip = True
 
-    def calculate_total_size( self):
+    def calculate_total_size(self):
         if self.size_attribute is not None:
             (total_size, count) = (0, 0)
             for episode in self.get_selected_episodes():
                 try:
-                    total_size += int(getattr( episode, self.size_attribute))
+                    total_size += int(getattr(episode, self.size_attribute))
                     count += 1
                 except:
                     pass
 
             text = []
-            if count == 0: 
+            if count == 0:
                 text.append(_('Nothing selected'))
-            text.append(N_('%(count)d episode', '%(count)d episodes', count) % {'count':count})
-            if total_size > 0: 
+            text.append(N_('%(count)d episode', '%(count)d episodes',
+                           count) % {'count': count})
+            if total_size > 0:
                 text.append(_('size: %s') % util.format_filesize(total_size))
             self.labelTotalSize.set_text(', '.join(text))
-            self.btnOK.set_sensitive(count>0)
-            self.btnRemoveAction.set_sensitive(count>0)
+            self.btnOK.set_sensitive(count > 0)
+            self.btnRemoveAction.set_sensitive(count > 0)
             if count > 0:
-                self.btnCancel.set_label(gtk.STOCK_CANCEL)
+                self.btnCancel.set_label(_('_Cancel'))
             else:
-                self.btnCancel.set_label(gtk.STOCK_CLOSE)
+                self.btnCancel.set_label(_('_Close'))
         else:
             self.btnOK.set_sensitive(False)
             self.btnRemoveAction.set_sensitive(False)
             for index, row in enumerate(self.model):
-                if self.model.get_value(row.iter, self.COLUMN_TOGGLE) == True:
+                if self.model.get_value(row.iter, self.COLUMN_TOGGLE) is True:
                     self.btnOK.set_sensitive(True)
                     self.btnRemoveAction.set_sensitive(True)
                     break
             self.labelTotalSize.set_text('')
 
-    def toggle_cell_handler( self, cell, path):
+    def toggle_cell_handler(self, cell, path):
         model = self.treeviewEpisodes.get_model()
         model[path][self.COLUMN_TOGGLE] = not model[path][self.COLUMN_TOGGLE]
 
@@ -342,21 +346,21 @@ class gPodderEpisodeSelector(BuilderWidget):
     def custom_selection_button_clicked(self, button, label):
         callback = self.selection_buttons[label]
 
-        for index, row in enumerate( self.model):
-            new_value = callback( self.episodes[index])
-            self.model.set_value( row.iter, self.COLUMN_TOGGLE, new_value)
+        for index, row in enumerate(self.model):
+            new_value = callback(self.episodes[index])
+            self.model.set_value(row.iter, self.COLUMN_TOGGLE, new_value)
 
         self.calculate_total_size()
 
-    def on_btnCheckAll_clicked( self, widget):
+    def on_btnCheckAll_clicked(self, widget):
         for row in self.model:
-            self.model.set_value( row.iter, self.COLUMN_TOGGLE, True)
+            self.model.set_value(row.iter, self.COLUMN_TOGGLE, True)
 
         self.calculate_total_size()
 
-    def on_btnCheckNone_clicked( self, widget):
+    def on_btnCheckNone_clicked(self, widget):
         for row in self.model:
-            self.model.set_value( row.iter, self.COLUMN_TOGGLE, False)
+            self.model.set_value(row.iter, self.COLUMN_TOGGLE, False)
 
         self.calculate_total_size()
 
@@ -385,12 +389,13 @@ class gPodderEpisodeSelector(BuilderWidget):
 
         self.calculate_total_size()
 
-    def get_selected_episodes( self, remove_episodes=False):
+    def get_selected_episodes(self, remove_episodes=False):
         selected_episodes = []
 
-        for index, row in enumerate( self.model):
-            if self.model.get_value( row.iter, self.COLUMN_TOGGLE) == True:
-                selected_episodes.append( self.episodes[self.model.get_value( row.iter, self.COLUMN_INDEX)])
+        for index, row in enumerate(self.model):
+            if self.model.get_value(row.iter, self.COLUMN_TOGGLE) is True:
+                selected_episodes.append(self.episodes[self.model.get_value(
+                    row.iter, self.COLUMN_INDEX)])
 
         if remove_episodes:
             for episode in selected_episodes:
@@ -404,13 +409,12 @@ class gPodderEpisodeSelector(BuilderWidget):
 
         return selected_episodes
 
-    def on_btnOK_clicked( self, widget):
+    def on_btnOK_clicked(self, widget):
         self.gPodderEpisodeSelector.destroy()
         if self.callback is not None:
-            self.callback( self.get_selected_episodes())
+            self.callback(self.get_selected_episodes())
 
-    def on_btnCancel_clicked( self, widget):
+    def on_btnCancel_clicked(self, widget):
         self.gPodderEpisodeSelector.destroy()
         if self.callback is not None:
             self.callback([])
-
