@@ -24,10 +24,10 @@
 #  Based on libpodcasts.py (thp, 2005-10-29)
 #
 
-import collections
 import datetime
 import glob
 import hashlib
+import json
 import logging
 import os
 import re
@@ -35,10 +35,10 @@ import shutil
 import string
 import time
 
-import gpodder
 import podcastparser
-from gpodder import (coverart, escapist_videos, feedcore, registry, schema,
-                     util, vimeo, youtube)
+
+import gpodder
+from gpodder import coverart, feedcore, registry, schema, util, vimeo, youtube
 
 logger = logging.getLogger(__name__)
 
@@ -110,13 +110,13 @@ class PodcastParserFeed(Feed):
     def get_link(self):
         vid = youtube.get_youtube_id(self.feed['url'])
         if vid is not None:
-            self.feed['link'] = youtube.get_channel_id_url(self.feed['url'])
+            self.feed['link'] = youtube.get_channel_id_url(self.feed['url'], self.fetcher.feed_data)
         return self.feed.get('link')
 
     def get_description(self):
         vid = youtube.get_youtube_id(self.feed['url'])
         if vid is not None:
-            self.feed['description'] = youtube.get_channel_desc(self.feed['url'])
+            self.feed['description'] = youtube.get_channel_desc(self.feed['url'], self.fetcher.feed_data)
         return self.feed.get('description')
 
     def get_cover_url(self):
@@ -148,20 +148,41 @@ class PodcastParserFeed(Feed):
         if self.max_episodes > 0 and len(entries) > self.max_episodes:
             entries = entries[:self.max_episodes]
 
+        num_duplicate_guids = 0
+
         # Search all entries for new episodes
         for entry in entries:
             episode = channel.EpisodeClass.from_podcastparser_entry(entry, channel)
             if episode is None:
                 continue
 
+            # Discard episode when its GUID collides with a newer episode
+            if episode.guid in seen_guids:
+                num_duplicate_guids += 1
+                channel._update_error = ('Discarded {} episode(s) with non-unique GUID, contact the podcast publisher to fix this issue.'
+                        .format(num_duplicate_guids))
+                logger.warning('Discarded episode with non-unique GUID, contact the podcast publisher to fix this issue. [%s] [%s]',
+                        channel.title, episode.title)
+                continue
+
             seen_guids.add(episode.guid)
             # Detect (and update) existing episode based on GUIDs
             existing_episode = existing_guids.get(episode.guid, None)
             if existing_episode:
+                if existing_episode.total_time == 0 and 'youtube' in episode.url:
+                    # query duration for existing youtube episodes that haven't been downloaded or queried
+                    # such as live streams after they have ended
+                    existing_episode.total_time = youtube.get_total_time(episode)
+
                 existing_episode.update_from(episode)
+                existing_episode.cache_text_description()
                 existing_episode.save()
                 continue
+            elif episode.total_time == 0 and 'youtube' in episode.url:
+                # query duration for new youtube episodes
+                episode.total_time = youtube.get_total_time(episode)
 
+            episode.cache_text_description()
             episode.save()
             new_episodes.append(episode)
         return new_episodes, seen_guids
@@ -171,37 +192,41 @@ class PodcastParserFeed(Feed):
             url = self.feed['paged_feed_next']
             logger.debug("get_next_page: feed has next %s", url)
             url = channel.authenticate_url(url)
-            res = self.fetcher.fetch(url, max_episodes=max_episodes)
-            if res.status == feedcore.UPDATED_FEED:
-                res.feed = PodcastParserFeed(res.feed, self.fetcher, max_episodes)
-            return res
+            return self.fetcher.fetch(url, autodiscovery=False, max_episodes=max_episodes)
         return None
 
 
 class gPodderFetcher(feedcore.Fetcher):
     """
-    This class extends the feedcore Fetcher with the gPodder User-Agent and the
-    Proxy handler based on the current settings in gPodder.
+    This class implements fetching a channel from custom feed handlers
+    or the default using podcastparser
     """
     def fetch_channel(self, channel, max_episodes):
         custom_feed = registry.feed_handler.resolve(channel, None, max_episodes)
         if custom_feed is not None:
             return custom_feed
+        # TODO: revisit authenticate_url: pass auth as kwarg
         # If we have a username or password, rebuild the url with them included
         # Note: using a HTTPBasicAuthHandler would be pain because we need to
         # know the realm. It can be done, but I think this method works, too
         url = channel.authenticate_url(channel.url)
-        res = self.fetch(url, channel.http_etag, channel.http_last_modified, max_episodes)
-        if res.status == feedcore.UPDATED_FEED:
-            res.feed = PodcastParserFeed(res.feed, self, max_episodes)
-        return res
+        return self.fetch(url, channel.http_etag, channel.http_last_modified, max_episodes=max_episodes)
 
     def _resolve_url(self, url):
         url = youtube.get_real_channel_url(url)
         url = vimeo.get_real_channel_url(url)
-        url = escapist_videos.get_real_channel_url(url)
         return url
 
+    def parse_feed(self, url, feed_data, data_stream, headers, status, max_episodes=0, **kwargs):
+        self.feed_data = feed_data
+        try:
+            feed = podcastparser.parse(url, data_stream)
+            feed['url'] = url
+            feed['headers'] = headers
+            return feedcore.Result(status, PodcastParserFeed(feed, self, max_episodes))
+        except ValueError as e:
+            raise feedcore.InvalidFeed('Could not parse feed: {url}: {msg}'.format(url=url, msg=e))
+
 
 # Our podcast model:
 #
@@ -242,12 +267,13 @@ class PodcastEpisode(PodcastModelObject):
     # In theory, Linux can have 255 bytes (not characters!) in a filename, but
     # filesystems like eCryptFS store metadata in the filename, making the
     # effective number of characters less than that. eCryptFS recommends
-    # 140 chars, we use 120 here (140 - len(extension) - len(".partial")).
+    # 140 chars, we use 120 here (140 - len(extension) - len(".partial.webm"))
+    # (youtube-dl appends an extension after .partial, ".webm" is the longest).
     # References: gPodder bug 1898, http://unix.stackexchange.com/a/32834
     MAX_FILENAME_LENGTH = 120  # without extension
-    MAX_FILENAME_WITH_EXT_LENGTH = 140 - len(".partial")  # with extension
+    MAX_FILENAME_WITH_EXT_LENGTH = 140 - len(".partial.webm")  # with extension
 
-    __slots__ = schema.EpisodeColumns
+    __slots__ = schema.EpisodeColumns + ('_download_error', '_text_description',)
 
     def _deprecated(self):
         raise Exception('Property is deprecated!')
@@ -256,8 +282,8 @@ class PodcastEpisode(PodcastModelObject):
     is_locked = property(fget=_deprecated, fset=_deprecated)
 
     def has_website_link(self):
-        return bool(self.link) and (self.link != self.url or
-                youtube.is_video_link(self.link))
+        return bool(self.link) and (self.link != self.url
+                or youtube.is_video_link(self.link))
 
     @classmethod
     def from_podcastparser_entry(cls, entry, channel):
@@ -265,32 +291,54 @@ class PodcastEpisode(PodcastModelObject):
         episode.guid = entry['guid']
         episode.title = entry['title']
         episode.link = entry['link']
-        episode.description = entry['description']
+        episode.episode_art_url = entry.get('episode_art_url')
+
+        # Only one of the two description fields should be set at a time.
+        # This keeps the database from doubling in size and reduces load time from slow storage.
+        # episode._text_description is initialized by episode.cache_text_description() from the set field.
+        # episode.html_description() returns episode.description_html or generates from episode.description.
         if entry.get('description_html'):
+            episode.description = ''
             episode.description_html = entry['description_html']
+        else:
+            episode.description = util.remove_html_tags(entry['description'] or '')
+            episode.description_html = ''
 
         episode.total_time = entry['total_time']
         episode.published = entry['published']
         episode.payment_url = entry['payment_url']
+        episode.chapters = None
+        if entry.get("chapters"):
+            episode.chapters = json.dumps(entry["chapters"])
 
         audio_available = any(enclosure['mime_type'].startswith('audio/') for enclosure in entry['enclosures'])
         video_available = any(enclosure['mime_type'].startswith('video/') for enclosure in entry['enclosures'])
+        link_has_media = False
+        if not (audio_available or video_available):
+            _url = episode.url
+            episode.url = util.normalize_feed_url(entry['link'])
+            # Check if any extensions (e.g. youtube-dl) support the link
+            link_has_media = registry.custom_downloader.resolve(None, None, episode) is not None
+            episode.url = _url
+        media_available = audio_available or video_available or link_has_media
 
+        url_is_invalid = False
         for enclosure in entry['enclosures']:
             episode.mime_type = enclosure['mime_type']
 
             # Skip images in feeds if audio or video is available (bug 979)
             # This must (and does) also look in Media RSS enclosures (bug 1430)
-            if episode.mime_type.startswith('image/') and (audio_available or video_available):
+            if episode.mime_type.startswith('image/') and media_available:
                 continue
 
             # If we have audio or video available later on, skip
-            # 'application/octet-stream' data types (fixes Linux Outlaws)
-            if episode.mime_type == 'application/octet-stream' and (audio_available or video_available):
+            # all 'application/*' data types (fixes Linux Outlaws and peertube feeds)
+            if episode.mime_type.startswith('application/') and media_available:
                 continue
 
             episode.url = util.normalize_feed_url(enclosure['url'])
             if not episode.url:
+                url_is_invalid = True
                 continue
 
             episode.file_size = enclosure['file_size']
@@ -299,9 +347,15 @@ class PodcastEpisode(PodcastModelObject):
         # Brute-force detection of the episode link
         episode.url = util.normalize_feed_url(entry['link'])
         if not episode.url:
-            return None
+            # The episode has no downloadable content.
+            # Set an empty URL so downloading will fail.
+            episode.url = ''
+            # Display an error icon if URL is invalid.
+            if url_is_invalid or (entry['link'] is not None and entry['link'] != ''):
+                episode._download_error = 'Invalid episode URL'
+            return episode
 
-        if any(mod.is_video_link(episode.url) for mod in (youtube, vimeo, escapist_videos)):
+        if any(mod.is_video_link(episode.url) for mod in (youtube, vimeo)):
             return episode
 
         # Check if we can resolve this link to a audio/video file
@@ -312,7 +366,14 @@ class PodcastEpisode(PodcastModelObject):
         if file_type is not None:
             return episode
 
-        return None
+        if link_has_media:
+            return episode
+
+        # The episode has no downloadable content.
+        # It is either a blog post or it links to a webpage with content accessible from shownotes title.
+        # Remove the URL so downloading will fail.
+        episode.url = ''
+        return episode
 
     def __init__(self, channel):
         self.parent = channel
@@ -325,8 +386,10 @@ class PodcastEpisode(PodcastModelObject):
         self.file_size = 0
         self.mime_type = 'application/octet-stream'
         self.guid = ''
+        self.episode_art_url = None
         self.description = ''
         self.description_html = ''
+        self.chapters = None
         self.link = ''
         self.published = 0
         self.download_filename = None
@@ -344,6 +407,9 @@ class PodcastEpisode(PodcastModelObject):
         # Timestamp of last playback time
         self.last_playback = 0
 
+        self._download_error = None
+        self._text_description = ''
+
     @property
     def channel(self):
         return self.parent
@@ -364,8 +430,8 @@ class PodcastEpisode(PodcastModelObject):
         # "Podcast Name - Title" and "Podcast Name: Title" -> "Title"
         for postfix in (' - ', ': '):
             prefix = self.parent.title + postfix
-            if (self.title.startswith(prefix) and
-                    len(self.title) - len(prefix) > LEFTOVER_MIN):
+            if (self.title.startswith(prefix)
+                    and len(self.title) - len(prefix) > LEFTOVER_MIN):
                 return self.title[len(prefix):]
 
         regex_patterns = [
@@ -384,14 +450,14 @@ class PodcastEpisode(PodcastModelObject):
 
         # "#001: Title" -> "001: Title"
         if (
-                not self.parent._common_prefix and
-                re.match('^#\d+: ', self.title) and
-                len(self.title) - 1 > LEFTOVER_MIN):
+                not self.parent._common_prefix
+                and re.match(r'^#\d+: ', self.title)
+                and len(self.title) - 1 > LEFTOVER_MIN):
             return self.title[1:]
 
-        if (self.parent._common_prefix is not None and
-                self.title.startswith(self.parent._common_prefix) and
-                len(self.title) - len(self.parent._common_prefix) > LEFTOVER_MIN):
+        if (self.parent._common_prefix is not None
+                and self.title.startswith(self.parent._common_prefix)
+                and len(self.title) - len(self.parent._common_prefix) > LEFTOVER_MIN):
             return self.title[len(self.parent._common_prefix):]
 
         return self.title
@@ -410,11 +476,80 @@ class PodcastEpisode(PodcastModelObject):
         if task is None:
             return False
 
-        return task.status in (task.DOWNLOADING, task.QUEUED, task.PAUSED)
+        return task.status in (task.DOWNLOADING, task.QUEUED, task.PAUSING, task.PAUSED, task.CANCELLING)
+
+    def get_player(self, config):
+        file_type = self.file_type()
+        if file_type == 'video' and config.player.video and config.player.video != 'default':
+            player = config.player.video
+        elif file_type == 'audio' and config.player.audio and config.player.audio != 'default':
+            player = config.player.audio
+        else:
+            player = 'default'
+        return player
+
+    def can_play(self, config):
+        """
+        # gPodder.playback_episodes() filters selection with this method.
+        """
+        return (self.was_downloaded(and_exists=True)
+                or self.can_preview()
+                or self.can_stream(config))
+
+    def can_preview(self):
+        return (self.downloading
+                and self.download_task.custom_downloader is not None
+                and self.download_task.custom_downloader.partial_filename is not None
+                and os.path.exists(self.download_task.custom_downloader.partial_filename))
+
+    def can_stream(self, config):
+        """
+        Don't try streaming if the user has not defined a player
+        or else we would probably open the browser when giving a URL to xdg-open.
+        We look at the audio or video player depending on its file type.
+        """
+        player = self.get_player(config)
+        return player and player != 'default'
+
+    def can_download(self):
+        """
+        gPodder.on_download_selected_episodes() filters selection with this method.
+        PAUSING and PAUSED tasks can be resumed.
+        """
+        return not self.was_downloaded(and_exists=True) and (
+            self.download_task is None
+            or self.download_task.can_queue()
+            or self.download_task.status == self.download_task.PAUSING)
+
+    def can_pause(self):
+        """
+        gPodder.on_pause_selected_episodes() filters selection with this method.
+        """
+        return self.download_task is not None and self.download_task.can_pause()
+
+    def can_cancel(self):
+        """
+        DownloadTask.cancel() only cancels the following tasks.
+        """
+        return self.download_task is not None and self.download_task.can_cancel()
+
+    def can_delete(self):
+        """
+        gPodder.delete_episode_list() filters out locked episodes, and cancels all unlocked tasks in selection.
+        """
+        return self.state != gpodder.STATE_DELETED and not self.archive and (
+            self.download_task is None or self.download_task.status == self.download_task.FAILED)
+
+    def can_lock(self):
+        """
+        gPodder.on_item_toggle_lock_activate() unlocks deleted episodes and toggles all others.
+        Locked episodes can always be unlocked.
+        """
+        return self.state != gpodder.STATE_DELETED or self.archive
 
     def check_is_new(self):
-        return (self.state == gpodder.STATE_NORMAL and self.is_new and
-                not self.downloading)
+        return (self.state == gpodder.STATE_NORMAL and self.is_new
+                and not self.downloading)
 
     def save(self):
         gpodder.user_extensions.on_episode_save(self)
@@ -460,10 +595,22 @@ class PodcastEpisode(PodcastModelObject):
 
     age_prop = property(fget=get_age_string)
 
+    def cache_text_description(self):
+        if self.description:
+            self._text_description = self.description
+        elif self.description_html:
+            self._text_description = util.remove_html_tags(self.description_html)
+        else:
+            self._text_description = ''
+
+    def html_description(self):
+        return self.description_html \
+            or util.nice_html_description(self.episode_art_url, self.description or _('No description available'))
+
     def one_line_description(self):
         MAX_LINE_LENGTH = 120
-        desc = util.remove_html_tags(self.description or '')
-        desc = re.sub('\s+', ' ', desc).strip()
+        desc = self._text_description
+        desc = re.sub(r'\s+', ' ', desc).strip()
         if not desc:
             return _('No description available')
         else:
@@ -481,6 +628,7 @@ class PodcastEpisode(PodcastModelObject):
             gpodder.user_extensions.on_episode_delete(self, filename)
             util.delete_file(filename)
 
+        self._download_error = None
         self.set_state(gpodder.STATE_DELETED)
 
     def get_playback_url(self, config=None, allow_partial=False):
@@ -492,15 +640,14 @@ class PodcastEpisode(PodcastModelObject):
         Also returns the filename of a partially downloaded file
         in case partial (preview) playback is desired.
         """
-        url = self.local_filename(create=False)
+        if (allow_partial and self.can_preview()):
+            return self.download_task.custom_downloader.partial_filename
 
-        if (allow_partial and url is not None and
-                os.path.exists(url + '.partial')):
-            return url + '.partial'
+        url = self.local_filename(create=False)
 
         if url is None or not os.path.exists(url):
             # FIXME: may custom downloaders provide the real url ?
-            url = registry.download_url.resolve(config, self.url, self)
+            url = registry.download_url.resolve(config, self.url, self, allow_partial)
         return url
 
     def find_unique_file_name(self, filename, extension):
@@ -508,8 +655,8 @@ class PodcastEpisode(PodcastModelObject):
         filename = filename.strip('.' + string.whitespace) + extension
 
         for name in util.generate_names(filename):
-            if (not self.db.episode_filename_exists(self.podcast_id, name) or
-                    self.download_filename == name):
+            if (not self.db.episode_filename_exists(self.podcast_id, name)
+                    or self.download_filename == name):
                 return name
 
     def local_filename(self, create, force_update=False, check_only=False,
@@ -553,7 +700,7 @@ class PodcastEpisode(PodcastModelObject):
         if not check_only and (force_update or not self.download_filename):
             # Avoid and catch gPodder bug 1440 and similar situations
             if template == '':
-                logger.warn('Empty template. Report this podcast URL %s',
+                logger.warning('Empty template. Report this podcast URL %s',
                         self.channel.url)
                 template = None
 
@@ -566,16 +713,15 @@ class PodcastEpisode(PodcastModelObject):
 
             if 'redirect' in episode_filename and template is None:
                 # This looks like a redirection URL - force URL resolving!
-                logger.warn('Looks like a redirection to me: %s', self.url)
+                logger.warning('Looks like a redirection to me: %s', self.url)
                 url = util.get_real_url(self.channel.authenticate_url(self.url))
                 logger.info('Redirection resolved to: %s', url)
                 episode_filename, _ = util.filename_from_url(url)
 
             # Use title for YouTube, Vimeo and Soundcloud downloads
-            if (youtube.is_video_link(self.url) or
-                    vimeo.is_video_link(self.url) or
-                    escapist_videos.is_video_link(self.url) or
-                    episode_filename == 'stream'):
+            if (youtube.is_video_link(self.url)
+                    or vimeo.is_video_link(self.url)
+                    or episode_filename == 'stream'):
                 episode_filename = self.title
 
             # If the basename is empty, use the md5 hexdigest of the URL
@@ -610,7 +756,7 @@ class PodcastEpisode(PodcastModelObject):
                     # call it from the downloading code before saving the file
                     logger.info('Choosing new filename: %s', new_file_name)
                 else:
-                    logger.warn('%s exists or %s does not', new_file_name, old_file_name)
+                    logger.warning('%s exists or %s does not', new_file_name, old_file_name)
                 logger.info('Updating filename of %s to "%s".', self.url, wanted_filename)
             elif self.download_filename is None:
                 logger.info('Setting download filename: %s', wanted_filename)
@@ -664,7 +810,7 @@ class PodcastEpisode(PodcastModelObject):
 
     def file_type(self):
         # Assume all YouTube/Vimeo links are video files
-        if youtube.is_video_link(self.url) or vimeo.is_video_link(self.url) or escapist_videos.is_video_link(self.url):
+        if youtube.is_video_link(self.url) or vimeo.is_video_link(self.url):
             return 'video'
 
         return util.file_type_by_extension(self.extension())
@@ -681,7 +827,7 @@ class PodcastEpisode(PodcastModelObject):
         try:
             return datetime.datetime.fromtimestamp(self.published).strftime('%H%M')
         except:
-            logger.warn('Cannot format pubtime: %s', self.title, exc_info=True)
+            logger.warning('Cannot format pubtime: %s', self.title, exc_info=True)
             return '0000'
 
     def playlist_title(self):
@@ -696,11 +842,18 @@ class PodcastEpisode(PodcastModelObject):
                 self.title,
                 self.cute_pubdate())
 
-    def cute_pubdate(self):
+    def cute_pubdate(self, show_time=False):
         result = util.format_date(self.published)
         if result is None:
             return '(%s)' % _('unknown')
-        else:
+
+        try:
+            if show_time:
+                timestamp = datetime.datetime.fromtimestamp(self.published)
+                return '<small>{}</small>\n{}'.format(timestamp.strftime('%H:%M'), result)
+            else:
+                return result
+        except:
             return result
 
     pubdate_prop = property(fget=cute_pubdate)
@@ -732,9 +885,10 @@ class PodcastEpisode(PodcastModelObject):
         current position is greater than 99 percent of the
         total time or inside the last 10 seconds of a track.
         """
-        return (self.current_position > 0 and self.total_time > 0 and
-                (self.current_position + 10 >= self.total_time or
-                 self.current_position >= self.total_time * .99))
+        return (self.current_position > 0
+                and self.total_time > 0
+                and (self.current_position + 10 >= self.total_time
+                or self.current_position >= self.total_time * .99))
 
     def get_play_info_string(self, duration_only=False):
         duration = util.format_time(self.total_time)
@@ -752,7 +906,8 @@ class PodcastEpisode(PodcastModelObject):
             return '-'
 
     def update_from(self, episode):
-        for k in ('title', 'url', 'description', 'description_html', 'link', 'published', 'guid', 'payment_url'):
+        for k in ('title', 'url', 'episode_art_url', 'description', 'description_html', 'chapters', 'link',
+                  'published', 'guid', 'payment_url'):
             setattr(self, k, getattr(episode, k))
         # Don't overwrite file size on downloaded episodes
         # See #648 refreshing a youtube podcast clears downloaded file size
@@ -761,7 +916,7 @@ class PodcastEpisode(PodcastModelObject):
 
 
 class PodcastChannel(PodcastModelObject):
-    __slots__ = schema.PodcastColumns + ('_common_prefix',)
+    __slots__ = schema.PodcastColumns + ('_common_prefix', '_update_error',)
 
     UNICODE_TRANSLATE = {ord('ö'): 'o', ord('ä'): 'a', ord('ü'): 'u'}
 
@@ -813,6 +968,8 @@ class PodcastChannel(PodcastModelObject):
             self.children = self.db.load_episodes(self, self.episode_factory)
             self._determine_common_prefix()
 
+        self._update_error = None
+
     @property
     def model(self):
         return self.parent
@@ -834,7 +991,7 @@ class PodcastChannel(PodcastModelObject):
             logger.debug('Strategy for %s changed to %s', self.title, caption)
             self.download_strategy = download_strategy
         else:
-            logger.warn('Cannot set strategy to %d', download_strategy)
+            logger.warning('Cannot set strategy to %d', download_strategy)
 
     def rewrite_url(self, new_url):
         new_url = util.normalize_feed_url(new_url)
@@ -873,6 +1030,9 @@ class PodcastChannel(PodcastModelObject):
 
                 known_files.add(filename)
 
+        # youtube-dl and yt-dlp create <name>.partial and <name>.partial.<ext> files while downloading.
+        # On startup, the latter is reported as an unknown external file.
+        # Both files are properly removed when the download completes.
         existing_files = set(filename for filename in
                 glob.glob(os.path.join(self.save_dir, '*'))
                 if not filename.endswith('.partial'))
@@ -880,8 +1040,8 @@ class PodcastChannel(PodcastModelObject):
         ignore_files = ['folder' + ext for ext in
                 coverart.CoverDownloader.EXTENSIONS]
 
-        external_files = existing_files.difference(list(known_files) +
-                [os.path.join(self.save_dir, ignore_file)
+        external_files = existing_files.difference(list(known_files)
+                + [os.path.join(self.save_dir, ignore_file)
                  for ignore_file in ignore_files])
         if not external_files:
             return
@@ -928,12 +1088,12 @@ class PodcastChannel(PodcastModelObject):
                         break
 
             if not found and not util.is_system_file(filename):
-                logger.warn('Unknown external file: %s', filename)
+                logger.warning('Unknown external file: %s', filename)
 
     @classmethod
     def sort_key(cls, podcast):
         key = util.convert_bytes(podcast.title.lower())
-        return re.sub('^the ', '', key).translate(cls.UNICODE_TRANSLATE)
+        return re.sub(r'^the ', '', key).translate(cls.UNICODE_TRANSLATE)
 
     @classmethod
     def load(cls, model, url, create=True, authentication_tokens=None, max_episodes=0):
@@ -987,11 +1147,13 @@ class PodcastChannel(PodcastModelObject):
 
         Returns: A new PodcastEpisode object
         """
-        return self.EpisodeClass.create_from_dict(d, self)
+        episode = self.EpisodeClass.create_from_dict(d, self)
+        episode.cache_text_description()
+        return episode
 
     def _consume_updated_title(self, new_title):
         # Replace multi-space and newlines with single space (Maemo bug 11173)
-        new_title = re.sub('\s+', ' ', new_title).strip()
+        new_title = re.sub(r'\s+', ' ', new_title).strip()
 
         # Only update the podcast-supplied title when we
         # don't yet have a title, or if the title is the
@@ -1075,7 +1237,7 @@ class PodcastChannel(PodcastModelObject):
                 next_feed = None
 
         # mark episodes not new
-        real_new_episode_count = 0
+        real_new_episodes = []
         # Search all entries for new episodes
         for episode in new_episodes:
             # Workaround for bug 340: If the episode has been
@@ -1087,17 +1249,18 @@ class PodcastChannel(PodcastModelObject):
                 episode.save()
 
             if episode.is_new:
-                real_new_episode_count += 1
+                real_new_episodes.append(episode)
 
             # Only allow a certain number of new episodes per update
-            if (self.download_strategy == PodcastChannel.STRATEGY_LATEST and
-                    real_new_episode_count > 1):
+            if (self.download_strategy == PodcastChannel.STRATEGY_LATEST
+                    and len(real_new_episodes) > 1):
                 episode.is_new = False
                 episode.save()
 
         self.children.extend(new_episodes)
 
         self.remove_unreachable_episodes(existing, seen_guids, max_episodes)
+        return real_new_episodes
 
     def remove_unreachable_episodes(self, existing, seen_guids, max_episodes):
         # Remove "unreachable" episodes - episodes that have not been
@@ -1105,8 +1268,8 @@ class PodcastChannel(PodcastModelObject):
         # Keep episodes that are currently being downloaded, though (bug 1534)
         if self.id is not None:
             episodes_to_purge = [e for e in existing if
-                    e.state != gpodder.STATE_DOWNLOADED and
-                    e.guid not in seen_guids and not e.downloading]
+                    e.state != gpodder.STATE_DOWNLOADED
+                    and e.guid not in seen_guids and not e.downloading]
 
             for episode in episodes_to_purge:
                 logger.debug('Episode removed from feed: %s (%s)',
@@ -1119,8 +1282,8 @@ class PodcastChannel(PodcastModelObject):
                     self.children.remove(episode)
 
         # This *might* cause episodes to be skipped if there were more than
-        # max_episodes_per_feed items added to the feed between updates.
-        # The benefit is that it prevents old episodes from apearing as new
+        # limit.episodes items added to the feed between updates.
+        # The benefit is that it prevents old episodes from appearing as new
         # in certain situations (see bug #340).
         self.db.purge(max_episodes, self.id)  # TODO: Remove from self.children!
 
@@ -1129,12 +1292,14 @@ class PodcastChannel(PodcastModelObject):
 
     def update(self, max_episodes=0):
         max_episodes = int(max_episodes)
+        new_episodes = []
         try:
             result = self.feed_fetcher.fetch_channel(self, max_episodes)
 
             if result.status == feedcore.UPDATED_FEED:
-                self._consume_updated_feed(result.feed, max_episodes)
+                new_episodes = self._consume_updated_feed(result.feed, max_episodes)
             elif result.status == feedcore.NEW_LOCATION:
+                # FIXME: could return the feed because in autodiscovery it is parsed already
                 url = result.feed
                 logger.info('New feed location: %s => %s', self.url, url)
                 if url in set(x.url for x in self.model.get_podcasts()):
@@ -1142,7 +1307,7 @@ class PodcastChannel(PodcastModelObject):
                 self.url = url
                 # With the updated URL, fetch the feed again
                 self.update(max_episodes)
-                return
+                return new_episodes
             elif result.status == feedcore.NOT_MODIFIED:
                 pass
 
@@ -1169,6 +1334,7 @@ class PodcastChannel(PodcastModelObject):
         self._determine_common_prefix()
 
         self.db.commit()
+        return new_episodes
 
     def delete(self):
         self.db.delete_podcast(self)
@@ -1198,7 +1364,7 @@ class PodcastChannel(PodcastModelObject):
         return self.section
 
     def _get_content_type(self):
-        if 'youtube.com' in self.url or 'vimeo.com' in self.url or 'escapistmagazine.com' in self.url:
+        if 'youtube.com' in self.url or 'vimeo.com' in self.url:
             return _('Video')
 
         audio, video, other = 0, 0, 0
@@ -1226,7 +1392,9 @@ class PodcastChannel(PodcastModelObject):
         if self.title == new_title:
             return
 
-        new_folder_name = self.find_unique_folder_name(new_title)
+        fn_template = util.sanitize_filename(new_title, self.MAX_FOLDERNAME_LENGTH)
+
+        new_folder_name = self.find_unique_folder_name(fn_template)
         if new_folder_name and new_folder_name != self.download_folder:
             new_folder = os.path.join(gpodder.downloads, new_folder_name)
             old_folder = os.path.join(gpodder.downloads, self.download_folder)
@@ -1274,13 +1442,12 @@ class PodcastChannel(PodcastModelObject):
         download_folder = download_folder.strip('.' + string.whitespace)
 
         for folder_name in util.generate_names(download_folder):
-            if (not self.db.podcast_download_folder_exists(folder_name) or
-                    self.download_folder == folder_name):
+            if (not self.db.podcast_download_folder_exists(folder_name)
+                    or self.download_folder == folder_name):
                 return folder_name
 
     def get_save_dir(self, force_new=False):
         if self.download_folder is None or force_new:
-            # we must change the folder name, because it has not been set manually
             fn_template = util.sanitize_filename(self.title, self.MAX_FOLDERNAME_LENGTH)
 
             if not fn_template:
